use clap::{Parser, Subcommand, ValueEnum};
use std::path::PathBuf;

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
pub struct Cli {
    #[command(subcommand)]
    pub command: Commands,
}

#[derive(ValueEnum, Debug, Copy, Clone, Eq, PartialEq)]
pub enum FormatMode {
    Pretty,
    Json,
    Compact,
}

impl std::fmt::Display for FormatMode {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        self.to_possible_value()
            .expect("no values are skipped")
            .get_name()
            .fmt(f)
    }
}

#[derive(Subcommand)]
pub enum Commands {
    /// Create an issue
    Create {
        /// The project key in which to create the issue
        #[arg(short, long)]
        project: Option<String>,

        /// Open the new issue in a browser
        #[arg(long)]
        open: bool,

        /// A Markdown file
        #[arg(value_name = "MARKDOWN_FILE")]
        markdown_file: Option<PathBuf>,
    },
    /// Find issues currently assigned to you
    List {
        /// Pick an output formatter
        #[arg(short, long, default_value_t = FormatMode::Pretty)]
        output: FormatMode,
    },
    /// Search for issues
    Search {
        /// Pick an output formatter
        #[arg(short, long, default_value_t = FormatMode::Pretty)]
        output: FormatMode,

        /// A JQL string
        #[arg(value_name = "JQL")]
        jql: String,
    },
    /// View an issue
    View {
        /// Pick an output formatter
        #[arg(short, long, default_value_t = FormatMode::Pretty)]
        output: FormatMode,

        /// An issue key, for example, KEY-123
        #[arg(value_name = "ISSUE")]
        issue: String,
    },
    /// Set up the configuration
    Init {
        /// Jira instance URL
        #[arg(long)]
        url: String,
        /// User email
        #[arg(long)]
        email: String,
        /// User Token
        #[arg(long)]
        token: String,
    },
}
